class PreProcessor(object):
    @staticmethod
    def parse(file, properties):
        f = open(file.get_url())
        stream = f.read()
        f.close()

        for key in properties.keys():
            if properties.get(key):
                stream = stream.replace("{" + key + "}", properties.get(key))

        return stream



import os


class File(object):
    def __init__(self, url=None):
        self.__url = url

    def get_url(self):
        return self.__url

    def is_file(self):
        url = self.get_url()
        return os.path.isfile(url)

    def is_dir(self):
        url = self.get_url()
        return os.path.isdir(url)

    def tail(self):
        url = self.get_url()
        head, tail = os.path.split(url)
        return tail

    def head(self):
        url = self.get_url()
        head, tail = os.path.split(url)
        return head

    def extension(self):
        url = self.get_url()
        filename, extension = os.path.splitext(url)
        return extension

    def exists(self):
        if self.is_file():
            return True
        if self.is_dir():
            return True
        return False



import noora

from noora.plugins.Plugin import Plugin


class HelpPlugin(Plugin):
    def __init__(self):
        Plugin.__init__(self, "help", None)

    def parse_args(self, parser, args):
        return parser.parse_args(args)

    def execute(self, arguments, properties):
        print(noora.__description__)
        print(noora.__uri__)
        print(noora.__version__)



from noora.connectors.Connector import Connector


class MysqlConnectorStub(Connector):
    def __init__(self):
        Connector.__init__(self)
        # FIXME: implement this upstream? MssqlConnector also uses it
        self.__result = None

    def get_result(self):
        return self.__result

    def set_result(self, result):
        self.__result = result

    def execute(self, executable, properties):
        pass



import os
import argparse

from noora.version.Versions import Versions
from noora.version.VersionLoader import VersionLoader

from noora.system.ClassLoader import ClassLoader

from noora.plugins.Plugin import Plugin

from noora.connectors.MssqlConnector import MssqlConnector


class RecreatePlugin(Plugin):
    def __init__(self):
        Plugin.__init__(self, "recreate", MssqlConnector())

    def parse_args(self, parser, args):
        parser.add_argument('-h', type=str, help='host', required=False)
        parser.add_argument('-s', type=str, help='schema', required=False)
        parser.add_argument('-e', type=str, help='environment', required=False)
        parser.add_argument('-v', type=str, help='version', required=False)
        parser.add_argument('-a', type=str, help='alias', required=False)
        return parser.parse_args(args)

    def find_plugin(self, properties, name):
        plugins = properties.get('plugins')
        for plugin in plugins:
            p = ClassLoader.find(plugin)
            if name.lower() == p.get_type().lower():
                return p

    def execute(self, arguments, properties):
        properties['create.dir'] = os.path.join(properties.get('current.dir'), 'create')
        properties['alter.dir'] = os.path.join(properties.get('current.dir'), 'alter')

        # find the drop plugin.
        plugin = self.find_plugin(properties, 'drop')

        # create the drop arguments and execute the plugin
        parser = argparse.ArgumentParser(add_help=False)
        parser.add_argument('-h', type=str, help='host', required=True)
        parser.add_argument('-s', type=str, help='schema', required=False)
        parser.add_argument('-e', type=str, help='environment', required=False)
        parser.add_argument('-a', type=str, help='alias', required=False)

        commands = []
        if arguments.h:
            commands.append('-h='+arguments.h)

        if arguments.s:
            commands.append('-s='+arguments.d)

        if arguments.e:
            commands.append('-e='+arguments.e)

        if arguments.a:
            commands.append('-a='+arguments.a)

        drop_arguments = parser.parse_args(commands)
        plugin.execute(drop_arguments, properties)

        # find the create plugin.
        plugin = self.find_plugin(properties, 'create')

        # create the create arguments and execute the plugin
        parser = argparse.ArgumentParser(add_help=False)
        parser.add_argument('-h', type=str, help='host', required=True)
        parser.add_argument('-s', type=str, help='schema', required=False)
        parser.add_argument('-e', type=str, help='environment', required=False)
        parser.add_argument('-a', type=str, help='alias', required=False)

        commands = []
        if arguments.h:
            commands.append('-h='+arguments.h)

        if arguments.s:
            commands.append('-s='+arguments.s)

        if arguments.e:
            commands.append('-e='+arguments.e)

        if arguments.a:
            commands.append('-a='+arguments.a)

        create_arguments = parser.parse_args(commands)
        plugin.execute(create_arguments, properties)

        versions = Versions()
        version_loader = VersionLoader(versions)
        version_loader.load(properties)
        versions.sort()

        # retrieve the update versions
        default_version = properties.get('default_version')
        update_versions = []
        for version in versions.list():
            if version.get_value() == default_version and arguments.v == version.get_value():
                break
            if version.get_value() == arguments.v:
                update_versions.append(version.get_value())
                break

            if version.get_value() != default_version:
                update_versions.append(version.get_value())

        # find the update plugin.
        plugin = self.find_plugin(properties, 'update')

        # create the arguments and execute the plugin
        for version in update_versions:
            parser = argparse.ArgumentParser(add_help=False)
            parser.add_argument('-h', type=str, help='host', required=True)
            parser.add_argument('-s', type=str, help='schema', required=False)
            parser.add_argument('-e', type=str, help='environment', required=False)
            parser.add_argument('-v', type=str, help='version', required=True)

            commands = []
            if arguments.h:
                commands.append('-h='+arguments.h)

            if arguments.s:
                commands.append('-s='+arguments.s)

            if arguments.e:
                commands.append('-e='+arguments.e)

            if arguments.a:
                commands.append('-a='+arguments.a)

            commands.append('-v='+version)

            update_arguments = parser.parse_args(commands)
            plugin.execute(update_arguments, properties)


import os

from noora.version.Version import Version


class VersionGuesser(object):
    def __init__(self, properties, versions):
        self.__properties = properties
        self.__versions = versions

    def guess(self, version):
        versions = self.__versions
        properties = self.__properties
        if version:
            return Version(version)
        if versions.next():
            return versions.next()
        return Version(properties.get("default_version"))

    def to_folder(self, version):
        properties = self.__properties
        if version == properties.get("default_version"):
            return properties.get("create.dir")
        return os.path.join(properties.get("alter.dir"), version)



from noora.io.File import File
from noora.io.Files import Files
from noora.version.Version import Version


class VersionLoader(object):
    def __init__(self, versions):
        self.__versions = versions

    def load(self, properties):
        alter = File(properties.get("alter.dir"))
        if alter.exists():
            files = Files()
            for version in files.list(alter):
                self.__versions.add(Version(version.tail()))

        create = File(properties.get("create.dir"))
        if create.exists():
            self.__versions.add(Version(properties.get("default_version")))



class Plugin(object):
    def __init__(self, type, connectable):
        self.__type = type
        self.__connectable = connectable
        self.__executable = None

    def set_connector(self, connectable):
        self.__connectable = connectable

    def get_connector(self):
        return self.__connectable

    def set_executor(self, executable):
        self.__executable = executable

    def get_executor(self):
        return self.__executable

    def set_type(self, type):
        self.__type = type

    def get_type(self):
        return self.__type



import os

from noora.version.Version import Version
from noora.version.Versions import Versions
from noora.version.VersionLoader import VersionLoader
from noora.exceptions.UnknownVersionException import UnknownVersionException

from noora.system import Ora
from noora.system import PropertyHelper
from noora.io.File import File

from noora.plugins.Plugin import Plugin
from noora.plugins.Fail import Fail

from noora.connectors.MssqlConnector import MssqlConnector
from noora.connectors.ConnectionExecutor import ConnectionExecutor


class UpdatePlugin(Plugin):
    def __init__(self):
        Plugin.__init__(self, "update", MssqlConnector())

    def parse_args(self, parser, args):
        parser.add_argument('-v', type=str, help='version', required=True)
        parser.add_argument('-h', type=str, help='host', required=True)
        parser.add_argument('-s', type=str, help='schema', required=False)
        parser.add_argument('-e', type=str, help='environment', required=False)

        return parser.parse_args(args)

    def fail_on_unknown_version(self, version, properties):
        alter_dir = properties.get('alter.dir')
        version_dir = File(os.path.join(alter_dir, version))
        if not version_dir.exists():
            raise UnknownVersionException("unknown version folder: {}".format(version))

    def fail_on_invalid_environment(self, connector, executor, environment, properties):
        plugin_dir = properties.get('plugin.dir')
        properties['environment'] = environment
        script = File(os.path.join(plugin_dir, 'mssql', 'update', 'checkenvironment.sql'))
        executor['script'] = script
        # FIXME: remove?
        # connector.execute(executor, properties)
        # if "(Code 1329)" in connector.get_result():
        #     raise InvalidEnvironmentException("invalid environment", environment)

    def fail_on_invalid_version(self, connector, executor, version, properties):
        plugin_dir = properties.get('plugin.dir')

        versions = Versions()
        version_loader = VersionLoader(versions)
        version_loader.load(properties)
        versions.sort()
        v = Version(version)
        previous = versions.previous(v).get_value()

        properties['previous'] = previous
        script = File(os.path.join(plugin_dir, 'mssql', 'update', 'checkversion.sql'))
        executor['script'] = script
        connector.execute(executor, properties)
        # if "(Code 1329)" in connector.get_result():
        #     raise InvalidVersionException("invalid version", previous)

    def execute(self, arguments, properties):
        properties['create.dir'] = os.path.join(properties.get('current.dir'), 'create')
        properties['alter.dir'] = os.path.join(properties.get('current.dir'), 'alter')

        host = arguments.h
        Fail.fail_on_no_host(host)

        version = arguments.v
        Fail.fail_on_no_version(version)

        default_schemes = properties.get('schemes')
        schemes = Ora.nvl(arguments.s, default_schemes)
        Fail.fail_on_invalid_schema(arguments.s, properties)

        default_environment = properties.get('default_environment')
        environment = Ora.nvl(arguments.e, default_environment)
        Fail.fail_on_invalid_environment(arguments.e, properties)

        database = properties.get('database')
        objects = properties.get('create_objects')

        version_schema = properties.get('version_schema')

        alter_dir = properties.get('alter.dir')

        self.fail_on_unknown_version(version, properties)

        # retrieve the user credentials for this database project.
        users = properties.get('mssql_users')

        # try to retrieve the users from the credentials file, when no users are configured in
        # myproject.json.
        if not users:
            # retrieve the name of this database project, introduced in version 1.0.12
            profile = PropertyHelper.get_profile(properties)
            if profile:
                users = profile.get('mssql_users')

        connector = self.get_connector()

        for schema in schemes:
            print("updating schema '{schema}' in database '{db}' "
                  "on host '{host}' using environment '{env}'".format(
                    schema=schema, db=database, host=host, env=environment))

            executor = PropertyHelper.get_mssql_properties(users, host, schema)
            executor['database'] = database

            # database_folder = PropertyHelper.get_database_folder(database, database_aliases)

            if schema == version_schema:
                self.fail_on_invalid_environment(connector, executor, environment, properties)
                self.fail_on_invalid_version(connector, executor, version, properties)

            for obj in objects:
                # global ddl objects
                folder = File(os.path.join(alter_dir, version, schema, 'ddl', obj))
                ConnectionExecutor.execute(connector, executor, properties, folder)

                # environment specific ddl objects
                folder = File(os.path.join(alter_dir, version, schema, 'ddl', obj, environment))
                ConnectionExecutor.execute(connector, executor, properties, folder)

            # global dat objects
            folder = File(os.path.join(alter_dir, version, schema, 'dat'))
            ConnectionExecutor.execute(connector, executor, properties, folder)

            # environment specific dat objects
            folder = File(os.path.join(alter_dir, version, schema, 'dat', environment))
            ConnectionExecutor.execute(connector, executor, properties, folder)

            print("database '{}' updated.".format(database))



import os

from noora.system import PropertyHelper
from noora.system import Ora
from noora.io.File import File

from noora.plugins.Plugin import Plugin
from noora.plugins.Fail import Fail
from noora.exceptions.BlockedHostException import BlockedHostException

from noora.connectors.ConnectionExecutor import ConnectionExecutor
from noora.connectors.MysqlConnector import MysqlConnector


class DropPlugin(Plugin):
    def __init__(self):
        Plugin.__init__(self, "drop", MysqlConnector())

    def parse_args(self, parser, args):
        parser.add_argument('-h', type=str, help='host', required=True)
        parser.add_argument('-d', type=str, help='database', required=False)
        parser.add_argument('-e', type=str, help='environment', required=False)
        parser.add_argument('-a', type=str, help='alias', required=False)

        return parser.parse_args(args)

    def get_drop_dir(self, properties):
        return os.path.join(properties.get('plugin.dir'), 'mysql', 'drop')

    def fail_on_blocked_hosts(self, host, properties):
        blocked_hosts = properties.get('blocked_hosts')
        if host in blocked_hosts:
            raise BlockedHostException("block host {}".format(host))

    def execute(self, arguments, properties):
        host = arguments.h
        Fail.fail_on_no_host(host)
        self.fail_on_blocked_hosts(host, properties)

        default_databases = properties.get('databases')
        databases = Ora.nvl(arguments.d, default_databases)
        Fail.fail_on_invalid_database(arguments.d, properties)

        default_environment = properties.get('default_environment')
        environment = Ora.nvl(arguments.e, default_environment)
        Fail.fail_on_invalid_environment(arguments.e, properties)

        objects = properties.get('drop_objects')

        alias = arguments.a
        Fail.fail_on_invalid_alias(arguments.a, properties)

        # if an alias is given, only the alias database will be installed, other databases will be
        # ignored.
        if alias:
            print("using alias : {}".format(alias))
            databases = [alias]

        # retrieve the user credentials for this database project.
        users = properties.get('mysql_users')

        # try to retrieve the users from the credentials file, when no users are configured in
        # myproject.json.
        if not users:
            # retrieve the name of this database project, introduced in version 1.0.12
            profile = PropertyHelper.get_profile(properties)
            if profile:
                users = profile.get('mysql_users')

        # fail when no users are found. This means that they are not set in myproject.json or
        # credentials.json
        Fail.fail_on_no_users(users)

        for database in databases:
            print("dropping database '{db}' on host '{host}' using environment '{env}'".format(
                db=database, host=host, env=environment))

            executor = PropertyHelper.get_mysql_properties(users, host, database)

            connector = self.get_connector()

            for obj in objects:
                folder = File(os.path.join(self.get_drop_dir(properties), obj))
                ConnectionExecutor.execute(connector, executor, properties, folder)

            print("database '{}' dropped".format(database))



import os
from os.path import expanduser
import argparse
import json

import noora
from noora.io.File import File
from noora.system.ClassLoader import ClassLoader


class App(object):
    def __init__(self):
        pass

    @staticmethod
    def get_parser():
        parser = argparse.ArgumentParser(
            description="mynoora, a sql deployment tool", add_help=False)
        parser.add_argument("commands", help="command to execute", type=str, nargs='+')
        parser.add_argument('-r', action='store_true', help='show the revision', required=False)
        return parser

    @staticmethod
    def get_config_file(properties):
        current_dir = properties.get("current.dir")
        project_file = properties.get("project.file")

        f = File(os.path.join(current_dir, project_file))
        if f.exists():
            return f

        noora_dir = properties.get("noora.dir")
        f = File(os.path.join(noora_dir, project_file))
        return f

    @staticmethod
    def properties():
        noora_dir = os.path.dirname(noora.__file__)
        current_dir = os.path.abspath('.')

        properties = dict()
        properties["noora.dir"] = noora_dir
        properties["current.dir"] = current_dir
        properties["plugin.dir"] = os.path.join(noora_dir, 'plugins')
        properties["project.file"] = "myproject.json"
        properties['home.dir'] = expanduser('~')
        return properties

    @staticmethod
    def load_properties(properties):
        f = App.get_config_file(properties)
        f = open(f.get_url())

        data = json.load(f)
        for key in data.keys():
            properties[key] = data[key]
        f.close()

    @staticmethod
    def find_plugin(command, properties):
        plugins = properties.get('plugins')
        for plugin in plugins:
            p = ClassLoader.find(plugin)
            if command.lower() == p.get_type().lower():
                return p

    @staticmethod
    def build_dir(version, properties):
        if version == properties.get("default_version"):
            return properties.get("create.dir")
        return os.path.join(properties.get("alter.dir"), version)



import subprocess

from noora.exceptions.PluginException import PluginException


class Shell(object):
    @staticmethod
    def execute(call):
        args = call['args']
        shell = call['shell']
        stdout = call['stdout']
        stderr = call['stderr']
        stdin = call['stdin']
        startupinfo = call['startupinfo']

        # TODO: Popen isn't used any more. Remove?
        #p = subprocess.Popen(args, shell=shell, stdin=stdin, stderr=stderr, startupinfo=startupinfo)

        result = subprocess.call(
            args, shell=True, stdin=stdin, stdout=stdout, stderr=stderr, startupinfo=startupinfo)

        #output = p.communicate()
        #if p.returncode != 0:

        if result != 0:
            f = open(stderr.name)
            stream = f.read()
            f.close()
            raise PluginException(stream)

        f = open(stdout.name)
        stream = f.read()
        f.close()

        return stream



import argparse

from noora.plugins.Plugin import Plugin


class GeneratePlugin(Plugin):
    def __init__(self):
        Plugin.__init__(self, "generate", None)

    def parse_args(self, parser, args):
        parser.add_argument('-t', type=str, help='technology', required=True)
        return parser.parse_args(args)

    def execute(self, arguments, properties):
        # TODO: fail on no technology
        # TODO: fail on invalid technology

        # resolve the generate plugin to execute based on the given technology
        technology = arguments.t

        plugin = None
        if technology == 'mysql':
            from noora.plugins.mysql.generate.GeneratePlugin import GeneratePlugin
            plugin = GeneratePlugin()
        if technology == 'mssql':
            from noora.plugins.mssql.generate.GeneratePlugin import GeneratePlugin
            plugin = GeneratePlugin()

        # no arguments are allowed here, the project generator is assumed.
        parser = argparse.ArgumentParser(add_help=False)
        parser.add_argument('-r', action='store_true', help='show the revision')
        parser.add_argument('-v', type=str, help='version', required=False)
        target_arguments = parser.parse_args([])

        plugin.execute(target_arguments, properties)




class ClassLoader(object):
    def __init__(self):
        pass

    @staticmethod
    def find(pattern):
        pattern_list = pattern.split(".")
        count = len(pattern_list)
        class_name = pattern_list[count - 1]
        module_name = ".".join(pattern_list[0:count - 1])

        mod = __import__(module_name, globals(), locals(), [''])
        clazz = getattr(mod, class_name)
        instance = clazz()
        return instance



from noora.exceptions.PluginException import PluginException
from noora.version.Versions import Versions
from noora.version.VersionLoader import VersionLoader
from noora.version.Version import Version


class Fail(object):
    @staticmethod
    def fail_on_no_host(host):
        if not host:
            raise PluginException("no host was given")

    @staticmethod
    def fail_on_no_version(version):
        if not version:
            raise PluginException("no version was given")

    @staticmethod
    def fail_on_no_command(commands):
        if not commands:
            raise PluginException("no command was given")

    @staticmethod
    def fail_on_no_users(users):
        if not users:
            raise PluginException("no users found")

    @staticmethod
    def fail_on_invalid_plugin(plugin):
        if not plugin:
            raise PluginException("the given plugin is not valid for this project")

    @staticmethod
    def fail_on_invalid_database(database, properties):
        if database:
            databases = properties.get('databases')
            if database not in databases:
                raise PluginException("the given database is not valid for this project")

    @staticmethod
    def fail_on_invalid_schema(schema, properties):
        if schema:
            schemes = properties.get('schemes')
            if schema not in schemes:
                raise PluginException("the given schema is not valid for this project")

    @staticmethod
    def fail_on_invalid_environment(environment, properties):
        if environment:
            environments = properties.get('environments')
            if environment not in environments:
                raise PluginException("the given environment is not valid for this project")

    @staticmethod
    def fail_on_invalid_alias(alias, properties):
        if alias:
            aliasses = properties.get('aliasses')
            if alias not in aliasses:
                raise PluginException("the given alias is not valid for this project")

    @staticmethod
    def fail_on_unknown_version(version, properties):
        versions = Versions()
        version_loader = VersionLoader(versions)
        version_loader.load(properties)
        versions.sort()

        if not versions.exists(Version(version)):
            raise PluginException("the given version is not valid for this project")



import os
import json
import shutil

from noora.version.Versions import Versions
from noora.version.VersionLoader import VersionLoader
from noora.version.VersionGuesser import VersionGuesser

from noora.system import Ora
from noora.io.File import File
from noora.io.Files import Files

from noora.plugins.Plugin import Plugin


class GeneratePlugin(Plugin):
    def __init__(self):
        Plugin.__init__(self, "generate", None)

    def parse_args(self, parser, args):
        parser.add_argument('-v', type=str, help='version', required=False)
        return parser.parse_args(args)

    def execute(self, arguments, properties):
        project = ''
        version = None

        if arguments.v:
            version = arguments.v

        current_dir = properties.get('current.dir')
        project_file = properties.get('project.file')
        config_file = File(os.path.join(current_dir, project_file))
        template_dir = os.path.join(
            properties.get('plugin.dir'), 'mysql', 'generate', 'templates')
        if not config_file.exists():
            database = input('database : ')
            project = database + "-db"
            host = input('host [localhost] : ')
            host = Ora.nvl(host, "localhost")
            port = input('port [3306] : ') or '3306'
            username = input('username : ')
            password = input('password : ')
            version = input('version [1.0.0]: ')
            version = Ora.nvl(version, "1.0.0")

            os.mkdir(project)
            template_file = os.path.join(template_dir, project_file)

            f = open(template_file)
            stream = f.read()
            f.close()
            stream = stream.replace('{project}', project)
            stream = stream.replace('{database}', database)
            stream = stream.replace('{host}', host)
            stream = stream.replace('{port}', port)
            stream = stream.replace('{username}', username)
            stream = stream.replace('{password}', password)
            stream = stream.replace('{version}', version)

            config_file = os.path.join(current_dir, project, project_file)
            f = open(config_file, 'w')
            f.write(stream)
            f.close()

        properties['alter.dir'] = os.path.join(current_dir, project, 'alter')
        properties['create.dir'] = os.path.join(current_dir, project, 'create')

        config_file = os.path.join(current_dir, project, project_file)
        f = open(config_file)
        data = json.load(f)
        for key in data.keys():
            properties[key] = data[key]
        f.close()

        versions = Versions()
        version_loader = VersionLoader(versions)
        version_loader.load(properties)
        versions.sort()
        version_guesser = VersionGuesser(properties, versions)
        next_version = version_guesser.guess(version).to_string()
        version_dir = version_guesser.to_folder(next_version)

        # create the version folder
        os.makedirs(version_dir)

        databases = properties.get('databases')
        version_database = properties.get('version_database')
        default_version = properties.get('default_version')
        environments = properties.get('environments')
        objects = properties.get('create_objects')

        for database in databases:
            # create the scheme folder
            database_dir = os.path.join(version_dir, database)
            os.mkdir(database_dir)

            # create the dat folder
            dat_dir = os.path.join(database_dir, 'dat')
            os.mkdir(dat_dir)

            # create the version script in the dat folder
            if database == version_database:
                version_file = os.path.join(dat_dir, 'version.sql')
                f = open(version_file, 'w')

                if next_version == default_version:
                    stream = properties.get('version_insert_statement')
                else:
                    stream = properties.get('version_update_statement')

                stream = stream.replace('<version>', next_version)
                f.write(stream)
                f.close()

                # sqlScript=self.getSqlVersionStatement(versions, version)
                # projectHelper.writeFile(datFolder+os.sep+'version.sql', sqlScript)

            # create the environment folders in the dat folder
            for environment in environments:
                os.mkdir(os.path.join(dat_dir, environment))

                # create the environment script in the dat folder.
                if database == version_database and next_version == default_version:
                    environment_file = os.path.join(dat_dir, environment, 'environment.sql')

                    f = open(environment_file, 'w')
                    stream = properties.get('environment_insert_statement')
                    stream = stream.replace('<environment>', environment)
                    f.write(stream)
                    f.close()

            # create the ddl folder
            ddl_dir = os.path.join(database_dir, 'ddl')
            os.mkdir(ddl_dir)

            # create the object folders in the ddl folder
            for obj in objects:
                os.mkdir(os.path.join(ddl_dir, obj))

            # create the template code on create.
            if database == version_database and next_version == default_version:
                for obj in objects:
                    object_dir = os.path.join(template_dir, obj)
                    if os.path.exists(object_dir):
                        files = Files.list(File(object_dir))
                        for file in files:
                            shutil.copyfile(
                                file.get_url(), os.path.join(ddl_dir, obj, file.tail()))

        print("version {} created.".format(next_version))



import os

from noora.system import PropertyHelper
from noora.system import Ora
from noora.io.File import File

from noora.plugins.Plugin import Plugin
from noora.plugins.Fail import Fail

from noora.connectors.MysqlConnector import MysqlConnector
from noora.connectors.ConnectionExecutor import ConnectionExecutor


class CreatePlugin(Plugin):
    def __init__(self):
        Plugin.__init__(self, "create", MysqlConnector())

    def parse_args(self, parser, args):
        parser.add_argument('-h', type=str, help='host', required=True)
        parser.add_argument('-d', type=str, help='database', required=False)
        parser.add_argument('-e', type=str, help='environment', required=False)
        parser.add_argument('-a', type=str, help='alias', required=False)

        return parser.parse_args(args)

    def execute(self, arguments, properties):
        properties['create.dir'] = os.path.join(properties.get('current.dir'), 'create')

        host = arguments.h
        Fail.fail_on_no_host(host)

        default_databases = properties.get('databases')
        databases = Ora.nvl(arguments.d, default_databases)
        Fail.fail_on_invalid_database(arguments.d, properties)

        default_environment = properties.get('default_environment')
        environment = Ora.nvl(arguments.e, default_environment)
        Fail.fail_on_invalid_environment(arguments.e, properties)

        objects = properties.get('create_objects')

        alias = arguments.a
        database_aliases = properties.get('database_aliases')
        Fail.fail_on_invalid_alias(alias, properties)

        # if an alias is given, only this database will be installed, other databases will be
        # ignored.
        if alias:
            print("using alias: {}".format(alias))
            databases = [alias]

        # retrieve the user credentials for this database project.
        users = properties.get('mysql_users')

        # try to retrieve the users from the credentials file, when no users are configured in
        # myproject.json.
        if not users:
            # retrieve the name of this database project, introduced in version 1.0.12
            profile = PropertyHelper.get_profile(properties)
            if profile:
                users = profile.get('mysql_users')

        # fail when no users are found. This means that they are not set in myproject.json or
        # credentials.json
        Fail.fail_on_no_users(users)

        connector = self.get_connector()
        create_dir = properties.get('create.dir')

        for database in databases:
            print("creating database '{db}' on host '{host}' using environment '{env}'".format(
                db=database, host=host, env=environment))

            executor = PropertyHelper.get_mysql_properties(users, host, database)

            database_folder = PropertyHelper.get_database_folder(database, database_aliases)

            for obj in objects:
                # global ddl objects
                folder = File(os.path.join(create_dir, database_folder, 'ddl', obj))
                ConnectionExecutor.execute(connector, executor, properties, folder)

                # environment specific ddl objects
                folder = File(os.path.join(create_dir, database_folder, 'ddl', obj, environment))
                ConnectionExecutor.execute(connector, executor, properties, folder)

            # global dat objects
            folder = File(os.path.join(create_dir, database_folder, 'dat'))
            ConnectionExecutor.execute(connector, executor, properties, folder)

            # environment specific dat objects
            folder = File(os.path.join(create_dir, database_folder, 'dat', environment))
            ConnectionExecutor.execute(connector, executor, properties, folder)

            print("database '{}' created.".format(database))



from noora.connectors.Connectable import Connectable
from noora.exceptions.ConnectorException import ConnectorException


class Connector(Connectable):
    def execute(self, executable, properties):
        raise ConnectorException("method not implemented")



import os

from noora.system import PropertyHelper
from noora.system import Ora
from noora.io.File import File

from noora.plugins.Plugin import Plugin
from noora.plugins.Fail import Fail

from noora.connectors.MssqlConnector import MssqlConnector
from noora.connectors.ConnectionExecutor import ConnectionExecutor


class CreatePlugin(Plugin):
    def __init__(self):
        Plugin.__init__(self, "create", MssqlConnector())

    def parse_args(self, parser, args):
        parser.add_argument('-h', type=str, help='host', required=True)
        parser.add_argument('-s', type=str, help='schema', required=False)
        parser.add_argument('-e', type=str, help='environment', required=False)
        return parser.parse_args(args)

    def execute(self, arguments, properties):
        properties['create.dir'] = os.path.join(properties.get('current.dir'), 'create')

        host = arguments.h
        Fail.fail_on_no_host(host)

        default_schemes = properties.get('schemes')
        schemes = Ora.nvl(arguments.s, default_schemes)
        Fail.fail_on_invalid_schema(arguments.s, properties)

        default_environment = properties.get('default_environment')
        environment = Ora.nvl(arguments.e, default_environment)
        Fail.fail_on_invalid_environment(arguments.e, properties)

        database = properties.get('database')
        objects = properties.get('create_objects')

        # retrieve the user credentials for this database project.
        users = properties.get('mssql_users')

        # try to retrieve the users from the credentials file, when no users are configured in
        # myproject.json.
        if not users:
            # retrieve the name of this database project, introduced in version 1.0.12
            profile = PropertyHelper.get_profile(properties)
            if profile:
                users = profile.get('mssql_users')

        connector = self.get_connector()
        create_dir = properties.get('create.dir')
        for schema in schemes:
            print("creating schema '{schema}' in database '{db}' "
                  "on host '{host}' using environment '{env}'".format(
                    schema=schema, db=database, host=host, env=environment))

            executor = PropertyHelper.get_mssql_properties(users, host, schema)
            executor['database'] = database

            for obj in objects:
                # global ddl objects
                folder = File(os.path.join(create_dir, schema, 'ddl', obj))
                ConnectionExecutor.execute(connector, executor, properties, folder)

                # environment specific ddl objects
                folder = File(os.path.join(create_dir, schema, 'ddl', obj, environment))
                ConnectionExecutor.execute(connector, executor, properties, folder)

            # global dat objects
            folder = File(os.path.join(create_dir, schema, 'dat'))
            ConnectionExecutor.execute(connector, executor, properties, folder)

            # environment specific dat objects
            folder = File(os.path.join(create_dir, schema, 'dat', environment))
            ConnectionExecutor.execute(connector, executor, properties, folder)

            print("schema '{}' created.".format(schema))



import os
import json
from copy import copy


# FIXME: write a proper docstring
def get_database_folder(database, folder_aliases):
    if folder_aliases:
        for folder_alias in folder_aliases:
            alias = folder_alias[0]
            if database == alias:
                return folder_alias[1]
    return database


def get_mssql_properties(users, host, schema):
    """
    For a list of user tuples, find the first occurrence that matches host and database.

    :param users: list containing a sequence of AT LEAST (host, schema, username, password)
    :param host: the hostname to match
    :param schema: the schema to match
    :return: a dict containing host, schema, username, password, port
    """
    for user in users:
        if user[0].lower() == host.lower() and user[1].lower() == schema.lower():
            res = {
                'host': host,
                'schema': schema,
                # Backwards compatibility with old project files
                'port': user[4] if len(user) > 4 else None,
                'username': user[2],
                'password': user[3],
            }
            return res

    return None


def get_mysql_properties(users, host, database):
    """
    For a list of user tuples, find the first occurrence that matches host and database.

    :param users: list containing a sequence of AT LEAST (host, database, username, password)
    :param host: the hostname to match
    :param database: the database to match
    :return: a dict containing host, database, username, password, port
    """
    for user in users:
        if user[0].lower() == host.lower() and user[1].lower() == database.lower():
            res = {
                'host': host,
                'database': database,
                # Backwards compatibility with old project files
                'port': user[4] if len(user) > 4 else None,
                'username': user[2],
                'password': user[3],
            }
            return res

    return None


def get_postgres_user(users, host, port, database):
    """
    For a list of host/port/database/user combinations, return the username that matches the
    host, port, database and user.

    :param users: list containing a sequence of AT LEAST (host, port, database, username)
    :param host: The hostname to match
    :param port: The port to match
    :param database: The database to match
    :return: A username or None
    """
    for user in users:
        if user[0].lower() == host.lower() and user[1].lower() == port.lower() and \
                user[2].lower() == database.lower():
            return user[3]
    return None


def get_oracle_user(users, sid, scheme):
    """
    For a list of sid/scheme/user combinations, return the username that matches the
    sid, scheme and user.

    :param users: list containing a sequence of AT LEAST (sid, scheme, username)
    :param sid: The Site Identifier (SID) to match
    :param scheme: The scheme to match
    :return: A username or None
    """
    for user in users:
        if user[0].lower() == sid.lower() and user[1].lower() == scheme.lower():
            return user[2]
    return None


def get_oracle_passwd(users, sid, scheme):
    """
    For a list of sid/scheme/user/password combinations, return the password that matches the
    sid, scheme and user.

    :param users: list containing a sequence of AT LEAST (sid, scheme, username, password)
    :param sid: The Site Identifier (SID) to match
    :param scheme: The scheme to match
    :return: A password or None
    """
    for user in users:
        if user[0].lower() == sid.lower() and user[1].lower() == scheme.lower():
            return user[3]
    return None


def get_profile(properties):
    """
    Retrieve the profile for the database project in the properties.
    """
    profile = None
    project = properties.get('project')
    if project:
        # load the users from the credentials file, if present
        home_dir = properties.get('home.dir')
        credentials_file = os.path.join(home_dir, '.noora', 'credentials.json')
        if os.path.isfile(credentials_file):
            f = open(credentials_file, 'r')
            credentials = json.load(f)
            f.close()

            profile = credentials.get(project)
    return profile



from noora.exceptions.ConnectorException import ConnectorException


class Connectable(object):
    def execute(self, executable, properties):
        raise ConnectorException("method not implemented")



#!/usr/bin/env python
import noora

from noora.system.App import App
from noora.plugins.Fail import Fail


def main(args=None):
    # retrieve the basic properties.
    properties = App.properties()

    # load the properties from myproject.json, in the current folder or in the noora folder
    App.load_properties(properties)

    # retrieve the basic mynoora parser.
    parser = App.get_parser()
    arguments, unknown = parser.parse_known_args(args)

    # show the revision
    if arguments.r:
        print(noora.__title__ + " version " + noora.__version__)
        exit(0)

    # execute the given commands
    commands = arguments.commands
    Fail.fail_on_no_command(commands)

    for command in commands:
        plugin = App.find_plugin(command, properties)
        Fail.fail_on_invalid_plugin(plugin)
        arguments = plugin.parse_args(parser, args)
        plugin.execute(arguments, properties)


if __name__ == "__main__":
    main(args=None)



__version__ = '1.1.0'

__title__ = 'noora'
__description__ = 'NoOra a database deployment tool.'
__uri__ = 'https://github.com/janripke/noora'
__doc__ = __description__ + ' <' + __uri__ + '>'

__author__ = 'Jan Ripke'
__email__ = 'jan.ripke@oxyma.nl'

__license__ = 'GNU General Public License v3 or later (GPLv3+)'
__copyright__ = 'Copyleft (l) 2017 Jan Ripke'

